@@ -9,6 +9,7 @@ use sxd_document::Package;
 use sxd_document::dom::*;
 use crate::errors::*;
 use regex::Regex;
+use crate::canonicalize::{name, as_element};
 
 
 use crate::navigate::*;
@@ -42,7 +43,14 @@ pub fn set_rules_dir(dir: String) -> Result<()> {
     let pref_manager = crate::prefs::PreferenceManager::get();
     return pref_manager.borrow_mut().initialize(PathBuf::from(dir));
 }
-/// The MathML to be spoken, brailled, or navigated.
+
+/// Returns the version number (from Cargo.toml) of the build
+pub fn get_version() -> String {
+    const VERSION: &str = env!("CARGO_PKG_VERSION");
+    return VERSION.to_string();
+}
+
+
 /// This will override any previous MathML that was set.
 /// This returns canonical MathML with 'id's set on any node that doesn't have an id.
 /// The ids can be used for sync highlighting if the `Bookmark` API preference is true.
@@ -115,20 +123,20 @@ pub fn get_overview_text() -> Result<String> {
 
 /// Get the value of the named preference.
 /// None is returned if `name` is not a known preference.
-pub fn get_preference(name: String) -> Option<String> {
+pub fn get_preference(name: String) -> Result<String> {
     use yaml_rust::Yaml;
     return crate::speech::SPEECH_RULES.with(|rules| {
         let rules = rules.borrow();
         let pref_manager = rules.pref_manager.borrow();
         let prefs = pref_manager.merge_prefs();
         return match prefs.get(&name) {
-            None => None,
+            None => bail!("No preference named '{}'", &name),
             Some(yaml) => match yaml {
-                Yaml::String(s) => Some(s.clone()),
-                Yaml::Boolean(b)  => Some( (if *b {"true"} else {"false"}).to_string() ),
-                Yaml::Integer(i)   => Some( format!("{}", *i)),
-                Yaml::Real(s)   => Some(s.clone()),
-                _                      => None,            
+                Yaml::String(s) => Ok(s.clone()),
+                Yaml::Boolean(b)  => Ok( (if *b {"true"} else {"false"}).to_string() ),
+                Yaml::Integer(i)   => Ok( format!("{}", *i)),
+                Yaml::Real(s)   => Ok(s.clone()),
+                _                       => bail!("Internal error in get_preference -- unknown YAML type"),            
             },
         }
     });
@@ -169,10 +177,10 @@ pub fn set_preference(name: String, value: String) -> Result<()> {
             if !pref_manager.get_api_prefs().to_string(&name).is_empty() {
                 match name.as_str() {
                     "Pitch" | "Rate" | "Volume" => {
-                        rules.pref_manager.borrow_mut().set_api_float_pref(&name, to_float(&name, &value)?);    
+                        pref_manager.set_api_float_pref(&name, to_float(&name, &value)?);    
                     },
                     "Bookmark" => {
-                        rules.pref_manager.borrow_mut().set_api_boolean_pref(&name, value.to_lowercase()=="true");    
+                        pref_manager.set_api_boolean_pref(&name, value.to_lowercase()=="true");    
                     },
                     _ => {
                         pref_manager.set_api_string_pref(&name, &value);
@@ -341,7 +349,7 @@ pub fn errors_to_string(e:&Error) -> String {
 }
 
 
-fn add_ids<'a>(mathml: Element<'a>) -> Element<'a> {
+fn add_ids(mathml: Element) -> Element {
     use std::time::SystemTime;
     let time = if cfg!(target_family = "wasm") {
         rand::random::<usize>()
@@ -354,7 +362,7 @@ fn add_ids<'a>(mathml: Element<'a>) -> Element<'a> {
     add_ids_to_all(mathml, &prefix, 0);
     return mathml;
 
-    fn add_ids_to_all<'a>(mathml: Element<'a>, id_prefix: &str, count: usize) -> usize {
+    fn add_ids_to_all(mathml: Element, id_prefix: &str, count: usize) -> usize {
         let mut count = count;
         if mathml.attribute("id").is_none() {
             mathml.set_attribute_value("id", (id_prefix.to_string() + &count.to_string()).as_str());
@@ -367,14 +375,14 @@ fn add_ids<'a>(mathml: Element<'a>) -> Element<'a> {
         }
         
         for child in mathml.children() {
-            let child = crate::canonicalize::as_element(child);
+            let child = as_element(child);
             count = add_ids_to_all(child, id_prefix, count);
         }
         return count;
     }
 }
 
-pub fn get_element<'a>(package: &'a Package) -> Element<'a> {
+pub fn get_element(package: &Package) -> Element {
     let doc = package.as_document();
     let mut result = None;
     for root_child in doc.root().children() {
@@ -425,7 +433,7 @@ pub fn trim_element(e: &Element) {
 
     // hack to avoid non-breaking whitespace from being removed -- move to a unique non-whitespace char then back
     const TEMP_NBSP: &str = "\u{F8FB}";
-    let trimmed_text = single_text.replace(" ", TEMP_NBSP).trim().replace(TEMP_NBSP, " ");
+    let trimmed_text = single_text.replace(' ', TEMP_NBSP).trim().replace(TEMP_NBSP, " ");
     if !e.children().is_empty() && !trimmed_text.is_empty() {
         // FIX: we have a problem -- what should happen???
         // FIX: For now, just keep the children and ignore the text and log an error -- shouldn't panic/crash
@@ -470,99 +478,122 @@ pub fn trim_element(e: &Element) {
 
 
 // used for testing trim
-// returns true if two Documents are equal
+/// returns Ok() if two Documents are equal or some info where they differ in the Err
 #[allow(dead_code)]
-fn is_same_doc(doc1: &Document, doc2: &Document) -> bool {
+fn is_same_doc(doc1: &Document, doc2: &Document) -> Result<()> {
+    // assume 'e' doesn't have element children until proven otherwise
+    // this means we keep Text children until we are proven they aren't needed
     if doc1.root().children().len() != doc2.root().children().len() {
-        return false;
+        bail!("Children of docs have {} != {} children", doc1.root().children().len(), doc2.root().children().len());
     }
-    for root_child in doc1.root().children().iter().zip(doc2.root().children().iter()) {
-        let (c1, c2) = root_child;
+
+    for (i, (c1, c2)) in doc1.root().children().iter().zip(doc2.root().children().iter()).enumerate() {
         match c1 {
             ChildOfRoot::Element(e1) => {
                 if let ChildOfRoot::Element(e2) = c2 {
-                    if is_same_element(e1, e2) {
-                        continue;
-                    }
+                    is_same_element(e1, e2)?;
+                } else {
+                    bail!("child #{}, first is element, second is something else", i);
                 }
-                return false;
             },
             ChildOfRoot::Comment(com1) => {
                 if let ChildOfRoot::Comment(com2) = c2 {
-                    if com1.text() == com2.text() {
-                        continue;
+                    if com1.text() != com2.text() {
+                        bail!("child #{} -- comment text differs", i);
                     }
+                } else {
+                    bail!("child #{}, first is comment, second is something else", i);
                 }
-                return false;
             }
             ChildOfRoot::ProcessingInstruction(p1) => {
                 if let ChildOfRoot::ProcessingInstruction(p2) = c2 {
-                    if p1.target() == p2.target() && p1.value() == p2.value() {
-                        continue;
+                    if p1.target() != p2.target() || p1.value() != p2.value() {
+                        bail!("child #{} -- processing instruction differs", i);
                     }
+                } else {
+                    bail!("child #{}, first is processing instruction, second is something else", i);
                 }
-                return false;
             }
         }
     };
-    return true;
+    return Ok( () );
 }
 
-/// Not really meant to be public -- used by tests in some packages
+/// returns Ok() if two Documents are equal or some info where they differ in the Err
+// Not really meant to be public -- used by tests in some packages
 #[allow(dead_code)]
-pub fn is_same_element(e1: &Element, e2: &Element) -> bool {
+pub fn is_same_element(e1: &Element, e2: &Element) -> Result<()> {
+    if name(e1) != name(e2) {
+        bail!("Names not the same: {}, {}", name(e1), name(e2));
+    }
+
     // assume 'e' doesn't have element children until proven otherwise
     // this means we keep Text children until we are proven they aren't needed
     if e1.children().len() != e2.children().len() {
-        return false;
+        bail!("Children of {} have {} != {} children", name(e1), e1.children().len(), e2.children().len());
     }
-    for element_child in e1.children().iter().zip(e2.children().iter()) {
-        let (c1, c2) = element_child;
+
+    if let Err(e) = attrs_are_same(e1.attributes(), e2.attributes()) {
+        bail!("In element {}, {}", name(e1), e);
+    }
+
+    for (i, (c1, c2)) in e1.children().iter().zip(e2.children().iter()).enumerate() {
         match c1 {
             ChildOfElement::Element(child1) => {
                 if let ChildOfElement::Element(child2) = c2 {
-                    if is_same_element(child1, child2) {
-                        continue;
-                    }
+                    is_same_element(child1, child2)?;
+                } else {
+                    bail!("{} child #{}, first is element, second is something else", name(e1), i);
                 }
-                return false;
             },
             ChildOfElement::Comment(com1) => {
                 if let ChildOfElement::Comment(com2) = c2 {
-                    if com1.text() == com2.text() {
-                        continue;
+                    if com1.text() != com2.text() {
+                        bail!("{} child #{} -- comment text differs", name(e1), i);
                     }
+                } else {
+                    bail!("{} child #{}, first is comment, second is something else", name(e1), i);
                 }
-                return false;
             }
             ChildOfElement::ProcessingInstruction(p1) => {
                 if let ChildOfElement::ProcessingInstruction(p2) = c2 {
-                    if p1.target() == p2.target() && p1.value() == p2.value() {
-                        continue;
+                    if p1.target() != p2.target() || p1.value() != p2.value() {
+                        bail!("{} child #{} -- processing instruction differs", name(e1), i);
                     }
+                } else {
+                    bail!("{} child #{}, first is processing instruction, second is something else", name(e1), i);
                 }
-                return false;
             }
             ChildOfElement::Text(t1) => {
                 if let ChildOfElement::Text(t2) = c2 {
-                    if t1.text() == t2.text() {
-                        continue;
+                    if t1.text() != t2.text() {
+                        bail!("{} child #{} --  text differs", name(e1), i);
                     }
-                    // debug!("#1 '{}[{}]', #2 '{}[{}]'", t1.text(), t1.text().len(),
-                    //         t2.text(), t2.text().len());
-                    // t1.text().chars().enumerate()
-                    //     .for_each(|(i, ch1)| {
-                    //         let ch2 = t2.text().chars().nth(i).unwrap();
-                    //         debug!("  {}: {}/{}, {}", i, ch1,  ch2, ch1==ch2)
-                    //     })
+                } else {
+                    bail!("{} child #{}, first is text, second is something else", name(e1), i);
                 }
-                return false;
             }
         }
     };
-    return true;
+    return Ok( () );
+
+    /// compares attributes -- '==' didn't seems to work
+    fn attrs_are_same(attrs1: Vec<Attribute>, attrs2: Vec<Attribute>) -> Result<()> {
+        if attrs1.len() != attrs2.len() {
+            bail!("Attributes have different length: {:?} != {:?}", attrs1, attrs2);
+        }
+        for (attr1, attr2) in attrs1.iter().zip(attrs2.iter()) {
+            if attr1.name().local_part() != attr2.name().local_part() || attr1.value() != attr2.value() {
+                bail!("Attribute {}.{} != {}.{}",
+                        attr1.name().local_part(), attr1.value(),
+                        attr2.name().local_part(), attr2.value());
+            }
+        }
+        return Ok( () );
+    }
 }
 
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -577,8 +608,11 @@ mod tests {
         let doc2 = package2.as_document();
         trim_doc(&doc2);
         debug!("doc2:\n{}", mml_to_string(&get_element(&package2)));
-            
-        is_same_doc(&doc1, &doc2)
+    
+        match is_same_doc(&doc1, &doc2) {
+			Ok(_) => return true,
+			Err(e) => panic!("{}", e),
+		}
     }
 
     #[test]
@@ -612,6 +646,18 @@ mod tests {
     fn trim_differs() {
         let whitespace_str = "<math> <mrow ><mo>-</mo><mi> a </mi></mrow ></math>";
         let different_str = "<math> <mrow ><mo>-</mo><mi> b </mi></mrow ></math>";
-        assert!(!are_parsed_strs_equal(different_str, whitespace_str));
+
+        // need to manually do this since failure shouldn't be a panic
+        let package1 = &parser::parse(whitespace_str).expect("Failed to parse input");
+        let doc1 = package1.as_document();
+        trim_doc(&doc1);
+        debug!("doc1:\n{}", mml_to_string(&get_element(&package1)));
+        
+        let package2 = parser::parse(different_str).expect("Failed to parse input");
+        let doc2 = package2.as_document();
+        trim_doc(&doc2);
+        debug!("doc2:\n{}", mml_to_string(&get_element(&package2)));
+
+        assert!(is_same_doc(&doc1, &doc2).is_err());
     }
 }
\ No newline at end of file
