@@ -10,6 +10,7 @@ use sxd_document::Package;
 use std::fmt;
 use crate::pretty_print::mml_to_string;
 use crate::speech::{NAVIGATION_RULES, CONCAT_INDICATOR, CONCAT_STRING};
+#[cfg(not(target_family = "wasm"))]
 use std::time::{Instant};
 use crate::errors::*;
 use crate::canonicalize::as_element;
@@ -53,7 +54,7 @@ impl fmt::Display for NavigationPosition {
     }
 }
 
-const ILLEGAL_NODE_ID: &'static str = "!not set";     // an illegal 'id' value
+const ILLEGAL_NODE_ID: &str = "!not set";     // an illegal 'id' value
 impl Default for NavigationPosition {
     fn default() -> Self {
         NavigationPosition {
@@ -125,9 +126,26 @@ impl NavigationState {
         self.position_stack.clear();
         self.command_stack.clear();
         self.where_am_i = NavigationPosition::default();
-        self.where_am_i_start_time;
+        self.reset_start_time()
+        
     }
 
+
+    // defining reset_start_time because of the following message if done inline
+    // attributes on expressions are experimental
+    // see issue #15701 <https://github.com/rust-lang/rust/issues/15701> for more information
+    #[cfg(target_family = "wasm")]
+    fn reset_start_time(&mut self) {
+         self.where_am_i_start_time = 0; // FIX: for web
+    }
+
+    #[cfg(not(target_family = "wasm"))]
+    fn reset_start_time(&mut self) {
+         self.where_am_i_start_time = Instant::now();      // need to give it some value, and "default()" isn't an option
+
+    }
+
+
     fn push(&mut self, position: NavigationPosition, command: &'static str) {
         self.position_stack.push(position);
         self.command_stack.push(command);
@@ -163,7 +181,7 @@ impl NavigationState {
         }
     }
 
-    pub fn get_navigation_mathml_id<'a>(&self, mathml: Element<'a>) -> (String, usize) {
+    pub fn get_navigation_mathml_id(&self, mathml: Element) -> (String, usize) {
         if self.position_stack.is_empty() {
             return (mathml.attribute_value("id").unwrap().to_string(), 0);
         } else {
@@ -177,12 +195,12 @@ impl NavigationState {
         context.set_variable("NavCommand", command);
 
         if command == "WhereAmI" && self.where_am_i == NavigationPosition::default() {
+            context.set_variable("NavNode", self.where_am_i.current_node.as_str());
+            context.set_variable("NavNodeOffset", self.where_am_i.current_node_offset as f64);
+        } else {
             let position = &self.position_stack[self.position_stack.len()-1];
             context.set_variable("NavNode", position.current_node.as_str());
             context.set_variable("NavNodeOffset", position.current_node_offset as f64);
-        } else {
-            context.set_variable("NavNode", self.where_am_i.current_node.as_str());
-            context.set_variable("NavNodeOffset", self.where_am_i.current_node_offset as f64);
         }
 
         // get the index from command (e.g., '3' in 'SetPlacemarker3 or MoveTo3' and set 'PlaceMarker' to it's position)
@@ -213,8 +231,8 @@ impl NavigationState {
 
         fn convert_last_char_to_number(str: &str) -> usize {
             let last_char = str.as_bytes()[str.len()-1];
-            assert!( '0' as u8 <= last_char && last_char <= '9' as u8);
-            return (last_char - '0' as u8) as usize;
+            assert!( (b'0'..=b'9').contains(&last_char) );
+            return (last_char - b'0') as usize;
         }
     }
 }
@@ -222,8 +240,8 @@ impl NavigationState {
 // convert the last digit of a Placemarker command to an integer
 fn convert_last_char_to_number(str: &str) -> usize {
     let last_char = str.as_bytes()[str.len()-1];
-    assert!( '0' as u8 <= last_char && last_char <= '9' as u8);
-    return (last_char - '0' as u8) as usize;
+    assert!( (b'0'..=b'9').contains(&last_char) );
+    return (last_char - b'0') as usize;
 }
 
 
@@ -268,7 +286,7 @@ pub fn context_get_variable<'c>(context: &Context<'c>, var_name: &str, mathml: E
                     };
                     let mut error_message = format!("Variable '{}' set somewhere in navigate.yaml is nodeset and not an attribute (correct by using '.../@id'??):\n", var_name);
                     if nodes.size() == 0 {
-                        error_message += &format!("0 nodes (false)");
+                        error_message += "0 nodes (false)";
                     } else {
                         let singular = nodes.size()==1;
                         error_message += &format!("{} node{}. {}:",
@@ -289,24 +307,24 @@ pub fn context_get_variable<'c>(context: &Context<'c>, var_name: &str, mathml: E
                     bail!(error_message);
                 },
             } ),
-            Err(__) => bail!("Could not find value for navigation variable '{}'", var_name),
+            Err(_) => bail!("Could not find value for navigation variable '{}'", var_name),
         }
     }
 }
 
 /// Given a key code along with the modifier keys, the current node is moved accordingly (or value reported in some cases).]
 /// The spoken text for the new current node is returned.
-pub fn do_mathml_navigate_key_press<'a>(mathml: Element<'a>,
+pub fn do_mathml_navigate_key_press(mathml: Element,
             key: usize, shift_key: bool, control_key: bool, alt_key: bool, meta_key: bool) -> Result<String> {
     let (command, param) = key_press_to_command_and_param(key, shift_key, control_key, alt_key, meta_key)?;
     return do_navigate_command_and_param(mathml, command, param);
 }
 
-fn do_navigate_command_and_param<'a>(mathml: Element<'a>, command: NavigationCommand, param: NavigationParam) -> Result<String> {
+fn do_navigate_command_and_param(mathml: Element, command: NavigationCommand, param: NavigationParam) -> Result<String> {
     return do_navigate_command_string(mathml, navigation_command_string(command, param));
 }
 
-pub fn do_navigate_command_string<'a>(mathml: Element<'a>, nav_command: &'static str) -> Result<String> {   
+pub fn do_navigate_command_string(mathml: Element, nav_command: &'static str) -> Result<String> {   
     // first check to see if nav file has been changed -- don't bother checking in loop below
     crate::speech::SpeechRules::update();
     NAVIGATION_RULES.with(|rules| { rules.borrow_mut().read_files() })?;
@@ -314,7 +332,7 @@ pub fn do_navigate_command_string<'a>(mathml: Element<'a>, nav_command: &'static
     // If no speech happened for some calls, we try the call the call again (e.g, no speech for invisible times).
     // To prevent to infinite loop, we limit the number of tries
     const LOOP_LIMIT: usize = 3;
-    static TRY_AGAIN: &'static str = "try again";
+    static TRY_AGAIN: &str = "try again";
     for loop_count in 0..LOOP_LIMIT {
         if mathml.children().is_empty() {
             bail!("MathML has not been set -- can't navigate");
@@ -342,21 +360,16 @@ pub fn do_navigate_command_string<'a>(mathml: Element<'a>, nav_command: &'static
 
                 nav_state.init_navigation_context(rules_with_context.get_context(), nav_command, nav_state.top());
                 
-                debug!("NavCommand: {}, NavMode {}", nav_command, nav_state.mode);
-
                 // start navigation off at the right node
-                let start_node_id = if nav_command == "MoveLastLocation" {
-                    match nav_state.pop() {
-                        None => mathml.attribute_value("id)").unwrap().to_string(),
-                        Some( (position, _) ) => position.current_node.clone(),
-                    }
-                } else {
-                    match nav_state.top() {
-                        None => mathml.attribute_value("id").unwrap().to_string(),
-                        Some( (position, _) ) => position.current_node.clone(),
-                    }
+                if nav_command == "MoveLastLocation" {
+                    nav_state.pop();
+                }
+                let start_node_id =  match nav_state.top() {
+                    None => mathml.attribute_value("id").unwrap().to_string(),
+                    Some( (position, _) ) => position.current_node.clone(),
                 };
-    
+                // debug!("NavCommand: {}, NavMode {}, start_node_id {}", nav_command, nav_state.mode, start_node_id);
+
                 let start_node = match get_node_by_id(mathml, &start_node_id) {
                     Some(node) => node,
                     None => {
@@ -373,7 +386,7 @@ pub fn do_navigate_command_string<'a>(mathml: Element<'a>, nav_command: &'static
                                                       .replace(CONCAT_INDICATOR, "")                            
                                             )
                             .trim());
-                debug!("Nav Speech: {}", speech);
+                // debug!("Nav Speech: {}", speech);
     
                 // FIX: add things that need to do
                 // do a speech replacement based on some marker for "where am i" and others that loop ([Speak: id])???
@@ -384,11 +397,11 @@ pub fn do_navigate_command_string<'a>(mathml: Element<'a>, nav_command: &'static
                 nav_state.mode = context_get_variable(context, "NavMode", mathml)?.0.unwrap();
                 rules.pref_manager.as_ref().borrow_mut().set_user_prefs("NavMode", &nav_state.mode);
 
-                let nav_position = match context_get_variable(&context, "NavNode", mathml)?.0 {
+                let nav_position = match context_get_variable(context, "NavNode", mathml)?.0 {
                     None => NavigationPosition::default(),
                     Some(node) => NavigationPosition {
                         current_node: node,
-                        current_node_offset: context_get_variable(&context, "NavNodeOffset", mathml)?.1.unwrap() as usize
+                        current_node_offset: context_get_variable(context, "NavNodeOffset", mathml)?.1.unwrap() as usize
                     }
                 };
     
@@ -405,22 +418,23 @@ pub fn do_navigate_command_string<'a>(mathml: Element<'a>, nav_command: &'static
     
                 if (nav_command.starts_with("Move") || nav_command.starts_with("Zoom")) && nav_command != "MoveLastLocation" {
                     // push the new location on the stack
-                    if &nav_position != &NavigationPosition::default() {
-                        debug!("nav_state: pushing on {}", &nav_position);
-                        if &nav_position.current_node != ILLEGAL_NODE_ID {
+                    if nav_position != NavigationPosition::default() {
+                        // debug!("nav_state: pushing on {}", &nav_position);
+                        let current_node = nav_position.current_node.as_str();
+                        if current_node != nav_state.top().unwrap().0.current_node && current_node != ILLEGAL_NODE_ID {
                             nav_state.push(nav_position.clone(), nav_command);
                         }
                     }
                 }
 
                 if nav_command.starts_with("SetPlacemarker") {
-                    if let Some(new_node_id) = context_get_variable(&context, "NavNode", mathml)?.0 {
-                        let offset = context_get_variable(&context, "NavNodeOffset", mathml)?.1.unwrap() as usize;
+                    if let Some(new_node_id) = context_get_variable(context, "NavNode", mathml)?.0 {
+                        let offset = context_get_variable(context, "NavNodeOffset", mathml)?.1.unwrap() as usize;
                         nav_state.place_markers[convert_last_char_to_number(nav_command)] = NavigationPosition{ current_node: new_node_id, current_node_offset: offset};
                     }
                 }
     
-                debug!("{}", &nav_state);
+                // debug!("{}", &nav_state);
                 let nav_mathml = get_node_by_id(mathml, &nav_position.current_node);
                 if nav_mathml.is_some() && context_get_variable(context, "SpeakExpression", mathml)?.0.unwrap() == "true" {
                     // Speak/Overview of where we landed (if we are supposed to speak it)
@@ -458,7 +472,7 @@ pub fn do_navigate_command_string<'a>(mathml: Element<'a>, nav_command: &'static
         let (top_position, top_command) = nav_state.pop().unwrap();
         let mut count = count-1;
         loop {
-            debug!("  ... loop count={}", count);
+            // debug!("  ... loop count={}", count);
             let (_, nav_command) = nav_state.top().unwrap();
             if (nav_command.starts_with("Move") || nav_command.starts_with("Zoom")) && nav_command != "MoveLastLocation" {
                 nav_state.pop();
@@ -902,8 +916,8 @@ mod tests {
             });
             test_command("ZoomOut", mathml, "msup");
 
-            let nav_speech = do_navigate_command_and_param(mathml, NavigationCommand::Zoom, NavigationParam::Previous)?;
-            debug!("Full speech: {}", nav_speech);
+            let _nav_speech = do_navigate_command_and_param(mathml, NavigationCommand::Zoom, NavigationParam::Previous)?;
+            // debug!("Full speech: {}", _nav_speech);
             NAVIGATION_STATE.with(|nav_stack| {
                 let (id, _) = nav_stack.borrow().get_navigation_mathml_id(mathml);
                 assert_eq!(id, "mfrac");
@@ -936,6 +950,37 @@ mod tests {
         });
     }
     
+    #[test]
+    fn text_extremes_and_move_last_location() -> Result<()> {
+        let mathml_str = "<math id='math'><mfrac id='mfrac'>
+                <msup id='msup'><mi id='base'>b</mi><mn id='exp'>2</mn></msup>
+                <mi id='denom'>d</mi>
+            </mfrac></math>";
+        crate::interface::set_rules_dir(super::super::abs_rules_dir_path()).unwrap();
+        set_mathml(mathml_str.to_string()).unwrap();
+        return MATHML_INSTANCE.with(|package_instance| {
+            let package_instance = package_instance.borrow();
+            let mathml = get_element(&*package_instance);
+            NAVIGATION_STATE.with(|nav_stack| {
+                nav_stack.borrow_mut().push(NavigationPosition{
+                    current_node: "base".to_string(),
+                    current_node_offset: 0
+                }, "None")
+            });
+
+            test_command("ZoomOutAll", mathml, "mfrac");
+            test_command("ZoomOut", mathml, "mfrac");
+            test_command("MoveLastLocation", mathml, "base");       // second zoom out should do nothing
+
+            test_command("ZoomOut", mathml, "msup");
+            test_command("ZoomInAll", mathml, "base");
+            test_command("ZoomIn", mathml, "base");
+            test_command("MoveLastLocation", mathml, "msup");       // second zoom in should do nothing
+
+            return Ok( () );
+        });
+    }
+    
     #[test]
     fn move_to_start() -> Result<()> {
         // init_logger();
@@ -964,8 +1009,8 @@ mod tests {
             });
             test_command("MoveLineStart", mathml, "msup");
 
-            let nav_speech = do_navigate_command_and_param(mathml, NavigationCommand::Move, NavigationParam::Start)?;
-            debug!("Full speech: {}", nav_speech);
+            let _nav_speech = do_navigate_command_and_param(mathml, NavigationCommand::Move, NavigationParam::Start)?;
+            // debug!("Full speech: {}", _nav_speech);
             NAVIGATION_STATE.with(|nav_stack| {
                 let (id, _) = nav_stack.borrow().get_navigation_mathml_id(mathml);
                 assert_eq!(id, "mfrac");
